function QC = ReadG7231Stream(FName, Frames)
% Read a byte stream file generated by a ITU-T G.7231 coder,
% returning the code values for selected frames.

% $Id: ReadG7231Stream.m,v 1.7 2020/11/19 02:20:16 pkabal Exp $

if (nargin < 2)
  Frames = [1, Inf];
end

FID = fopen(FName);
if (FID == -1)
  error('Cannot open bitstream file: %s', FullName(FName));
end
fprintf('G.723.1 Bitstream file: %s\n', FullName(FName));

IFr = 1;
k = 0;
while (IFr <= Frames(2))

  v = fread(FID, 1, 'uchar');
  if (isempty(v))
    break;
  end
  BT(1) = v;

  % Check the bit rate bits to determine the frame size (bytes)
  v = bitand(BT(1), 3);
  switch (v)
    case 0    % High rate
      N = 24;
    case 1    % Low rate
      N = 20;
    case 2    % SID frame
      N = 4;
    otherwise % Untransmitted frame
      N = 1;
  end

  if (N > 1)
    Data = fread(FID, N - 1, 'uint8');
    if (length(Data) ~= N - 1)
      error('Invalid bitstream file');
    end
    BT(2:N) = Data;
  end

  if (IFr >= Frames(1))
    k = k + 1;
    ByteStream{k} = BT(1:N);
  end

  IFr = IFr + 1;
end

fclose(FID);

% Convert to codes
QC = GetQCodes(ByteStream);

end
