function a = LSFxLP(lsf, LSFpar)
% Convert LSF's to LP coefficients. Optionally modify the LSF values to account
% for the use of linear interpolation of a cosine table as used in the G.723.1
% procedure.
%   lsf: Column vectors of LSF parameters

% $Id: LSFxLP.m,v 1.3 2020/11/03 18:11:47 pkabal Exp $

% In G.723.1, the conversion from LSFs to LP coefficients is carried out in the
% x = cos(w) domain. The conversion to the x-domain is done via linear
% interpolation into a cosine table. Here we modify the LSF's to take into
% account the error in the conversion to x values.
if (isfield(LSFpar.Fix, 'CosTable'))
  NCos = length(LSFpar.Fix.CosTable);
  w = (0:(NCos-1)) * 2 * pi / NCos;     % w runs from 0 to 2*pi
  x = interp1(w, LSFpar.Fix.CosTable, lsf, 'linear');
  lsf = acos(x);
end

% Convert to LP coefficients
N = size(lsf, 2);
M = size(lsf, 1);
a = NaN(M+1, N);
for i = 1:N
  ai = lsf2poly(lsf(:,i));    % Inconsistent: aI is a row vector
  a(:,i) = ai(:);
end

end
