function CGC = CodeGain(ACBbIB, Pulseval, Pitchpar)
% Combined codes for the adaptive codebook gains and the multipulse gains.

% $Id: CodeGain.m,v 1.6 2020/11/19 02:21:23 pkabal Exp $

GModV = Pitchpar.GModV;

NSubframe = length(Pulseval);
CGC = NaN(1, NSubframe);
for i = 1:NSubframe

  % Code the gains for the pitch and the multipulse contributions
  ShiftC = (~isinf(Pulseval(i).ShiftLag));
  CGC(i) = SubframeGainC(ACBbIB(:,i), Pulseval(i).gC, ShiftC, GModV);

end

end

%--------------------
function CGC = SubframeGainC(ACBbIB, gC, ShiftC, GModV)
% Form the combined gain code for the adaptive codebook gain and the
% multipulse gain.
% ACBbIB: Adaptive codebook gain (index, codebook). The index takes on
%         85 levels (codebook 1) or 170 levels (codebook 2)
% gC:     Multipulse amplitude code code (24 levels)
% ShiftC: Multipulse shift flag for shift and repeat operation. The use
%         of shift and repeat is only available if the pitch filter uses
%         codebook 1.

% The total number of combinations is 4080, which can be coded with 12 bits

ACBbC = ACBbIB(1) - 1;
CBookI = ACBbIB(2);
if (CBookI == 1)    % Choose 85 or 170 levels
  CGC = CombineVals([gC, ACBbC, ShiftC], GModV{CBookI});
else
  CGC = CombineVals([gC, ACBbC], GModV{CBookI});
end

end
