function tG7231CodeDecode()

% This test exercises the bit file and data file outputs of G7231Coder and
% the bit file and data file inputs of G7231Decoder

% $Id: tG7231CodeDecode.m,v 1.10 2020/12/08 13:16:38 pkabal Exp $

TDir = 'TestFiles';
WaveFileIn = fullfile(TDir, 'FE27_04.wav');
BitFile = 'FE27_04.bit';
BitFileRef = fullfile(TDir, 'FE27_04_Mref63.bit');
DataFile = 'FE27_04.dat';
DataFileRef = fullfile(TDir, 'FE27_04_Mref63.dat');
WaveFileOutB = 'FE27_04B.wav';
WaveFileOutD = 'FE27_04D.wav';

savepath = addpath('..');

SysOS = true;
CheckDiaryFile = StartDiaryFile(SysOS);

fprintf('\n----- Wave to Bit -----\n');
G7231Coder(WaveFileIn, BitFile);

fprintf('\n----- Compare Bit Files -----\n');
CompBitFiles(BitFile, BitFileRef);

fprintf('----- Wave to Data -----\n');
G7231Coder(WaveFileIn, DataFile);

fprintf('\n----- Compare Data Files -----\n');
Compare_DATFiles(DataFile, DataFileRef);

fprintf('\n----- Bit File to Wave File -----\n');
G7231Decoder(BitFile, WaveFileOutB);
delete(BitFile);

fprintf('\n----- Data File to Wave File -----\n');
G7231Decoder(DataFile, WaveFileOutD);
delete(DataFile);

fprintf('\n ----- Compare wave files from bit and data files\n');
CompAudioData(WaveFileOutB, WaveFileOutD);

delete(WaveFileOutB);
delete(WaveFileOutD);

CheckDiaryFile();

path(savepath);

end

% ----- -----
function Compare_DATFiles(DFile1, DFile2)

% Each data file contains a variable VC, with fields FMode, a, L, b, em
S1 = load(DFile1, '-mat');    % Structure S1, with field S1.VC
S2 = load(DFile2, '-mat');    % Structure S2, with field S2.VC

fprintf('Data file: %s\n', FullName(DFile1));
fprintf('Data file: %s\n', FullName(DFile2));

N1 = length(S1.VC);
N2 = length(S2.VC);

if (length(S1) ~= length(S2) || N1 ~= N2)
  error('Data files not compatible');
end

ErrA = 0;
iStart = NaN;    % Start of run
for i = 1:N1
  ErrType = Compare_DataFrame(S1.VC(i), S2.VC(i));
  if (any(ErrType))
    ErrA = 1;
    if (isnan(iStart))
      iStart = i;    % Start of run
      ErrTypeP = ErrType;
    elseif (any(ErrType ~= ErrTypeP))
      Print_Run(iStart, i, ErrType);
      iStart = NaN;
      ErrTypeP = ErrType;
    end
  else
    if (~isnan(iStart))
      Print_Run(iStart, i-1, ErrTypeP);
      iStart = NaN;
    end
  end
end
if (~isnan(iStart))
  Print_Run(iStart, N1, ErrTypeP);
end

if (~ErrA)
  fprintf('>>> No differences found\n');
end

end

% ----- -----
function ErrType = Compare_DataFrame(VC1, VC2)

k = 0;

k = k+1;
ErrType(k) = (VC1.FType ~= VC2.FType);
k = k +1;
ErrType(k) = any(VC1.a(:) ~= VC2.a(:));
k = k+1;
ErrType(k) = any(VC1.L ~= VC2.L);
k = k+1;
ErrType(k) = any(VC1.b(:) ~= VC2.b(:));
k = k+1;
ErrType(k) = any(VC1.em(:) ~= VC2.em(:));

end

% ----- -----
function Print_Run(iStart, iEnd, ErrType)

if (iEnd > iStart)
  fprintf('Frame: %d -> %d\n', iStart, iEnd);
else
  fprintf('Frame: %d\n', iStart);
end

ErrList = {'FType', 'a', 'L', 'em'};
NErrType = length(ErrType);
for k = 1:NErrType
  if (ErrType(k) ~= 0)
    fprintf('   %s differs\n', ErrList{k});
  end
end

end
