function tG7231Coder()
% Test G.723.1 coding

% $Id: tG7231Coder.m,v 1.17 2020/12/07 21:32:35 pkabal Exp $

% - This coder accesses files that were created by the reference coder/decoder.
% - The reference coder/decoder was compiled under Cygwin (Unix subsystem
%   running under Windows).

% $Id: tG7231Coder.m,v 1.17 2020/12/07 21:32:35 pkabal Exp $

SavePath = addpath('..', fullfile('..', 'Audio'), fullfile('..', 'Misc'));

SysOS = true;
CheckDiaryFile = StartDiaryFile(SysOS);

TDir = 'TestFiles';
tCoder(fullfile(TDir, 'MA01_02.wav'), fullfile(TDir, 'MA01_02_Mref63.bit'), ...
       fullfile(TDir, 'MA01_02_Cref63.bit'));

% ----------
CheckDiaryFile();

path(SavePath);

end

% ----- -----
function tCoder(FNameIn, FNameMbitref, FNameCbitref)

% 1. Run the Matlab coder, creating a bit file
% 2. Compare the Matlab generated bit file with the first reference bit file
% 3. If the C reference bit file is available
%    a) Compare the Matlab/C bit files
%    b) Run the Matlab decoder on both bit files
%    c) Compare the resulting WAVE files

if (nargin < 3)
  FNameCbitref = '';
elseif (nargin < 2)
  FNameMbitref = '';
end

% Form the file names
[~, Name] = fileparts(FNameIn);
FNameMbit = [Name '_M63.bit'];
FNameMwav = [Name '_M63.wav'];
FNameCwav = [Name '_Cref63.wav'];

% Run the Matlab coder
fprintf('====== Running the Matlab coder\n');
G7231Coder(FNameIn, FNameMbit);

% Compare the bit files
if (~isempty(FNameMbitref))
  fprintf('\n ------ Comparing Matlab generated bit files\n');
  CompBitFiles(FNameMbit, FNameMbitref);
  fprintf(' *** Expect no differences ***\n');
end

if (~isempty(FNameCbitref))

  fprintf(' ----- ----- ----- ----- ----- ----- -----\n');
  % Compare the the bit files
  fprintf('\n ------ Comparing Matlab and C generated bit files\n');
  fprintf(' *** Expect differences ***\n');
  CompBitFiles(FNameMbit, FNameCbitref);
  fprintf(' *** Expect differences ***\n\n');

  % Run the Matlab decoder on both bit files
  fprintf('====== Run the Matlab decoder\n');
  fprintf('\n ----- Matlab decoder with Matlab generated bit file\n');
  G7231Decoder(FNameMbit, FNameMwav);

  fprintf('\n ----- Matlab decoder with C-code generated bit file\n');
  G7231Decoder(FNameCbitref, FNameCwav);

  fprintf('\n ----- Compare audio files from Matlab and C bit files\n');
  CompAudioData(FNameCwav, FNameMwav);
  delete(FNameCwav);
  delete(FNameMwav);

end

delete(FNameMbit);

end
